MODULE trcsubstp
   !!======================================================================
   !!                       ***  MODULE trcsubstp  ***
   !! Time-stepping    : time loop of opa for passive tracer
   !!======================================================================
   !! History :  1.0  !  2004-03  (C. Ethe)  Original
   !!----------------------------------------------------------------------
#if defined key_top
   !!----------------------------------------------------------------------
   !!   trc_sub_stp     : passive tracer system sub-stepping 
   !!----------------------------------------------------------------------
   USE oce_trc          ! ocean dynamics and active tracers variables
   USE trc
   USE prtctl_trc       ! Print control for debbuging
   USE iom
   USE in_out_manager
#if defined key_zdftke
   USE zdftke          ! twice TKE (en)
#endif
#if defined key_zdfgls
   USE zdfgls, ONLY: en
#endif
   USE zdf_oce
   USE domvvl
   USE sshwzv_trc
   USE sbcrnf, ONLY:h_rnf

   IMPLICIT NONE

   PUBLIC   trc_sub_stp    ! called by trc_stp
   PUBLIC   trc_sub_stp_ini ! called by trc_ini to initialize substepping arrays.
   PUBLIC   trc_sub_stp_reset  !called by trc_stp to reset physics variables
#  include "domzgr_substitute.h90"
   
   !!----------------------------------------------------------------------
   !! NEMO/TOP 3.3 , NEMO Consortium (2010)
   !! $Id: trcstp.F90 2528 2010-12-27 17:33:53Z rblod $ 
   !! Software governed by the CeCILL licence (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE trc_sub_stp( kt )
      !!-------------------------------------------------------------------
      !!                     ***  ROUTINE trc_stp  ***
      !!                      
      !! ** Purpose : Average variables needed for sub-stepping passive tracers
      !! 
      !! ** Method  : 
      !!              Compute the averages for sub-stepping
      !!-------------------------------------------------------------------
      INTEGER, INTENT( in ) ::  kt  ! ocean time-step index
      INTEGER::				    ji,jj,jk
      !!-------------------------------------------------------------------

       IF (kt .eq. nit000) THEN
           sshb_hold=sshn
           sshu_b_hold=sshu_n
           sshv_b_hold=sshv_n
           IF(lwp) WRITE(numout,*)
           IF(lwp) WRITE(numout,*) 'trc_sub_stp : substepping of the passive tracers'
           IF(lwp) WRITE(numout,*) '~~~~~~~~~~~'
       ENDIF  
       IF( MOD( kt , nn_dttrc ) /= 0 ) THEN
         if(lwp)write(numout,*)'trc_sub_stp at kt=',kt
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  un_tm(ji,jj,jk)= un_tm(ji,jj,jk)+un(ji,jj,jk)*fse3u(ji,jj,jk)
                  vn_tm(ji,jj,jk)= vn_tm(ji,jj,jk)+vn(ji,jj,jk)*fse3v(ji,jj,jk)
		  wn_tm(ji,jj,jk)= wn_tm(ji,jj,jk)+wn(ji,jj,jk)*fse3w(ji,jj,jk)
                  avt_tm(ji,jj,jk)= avt_tm(ji,jj,jk)+avt(ji,jj,jk)*fse3t(ji,jj,jk)
#if defined key_ldfslp
		  wslpi_tm(ji,jj,jk)= wslpi_tm(ji,jj,jk)+wslpi(ji,jj,jk)*fse3w(ji,jj,jk)
		  wslpj_tm(ji,jj,jk)= wslpj_tm(ji,jj,jk)+wslpj(ji,jj,jk)*fse3w(ji,jj,jk)
		  uslp_tm(ji,jj,jk)= uslp_tm(ji,jj,jk)+uslp(ji,jj,jk)*fse3u(ji,jj,jk)
		  vslp_tm(ji,jj,jk)= vslp_tm(ji,jj,jk)+vslp(ji,jj,jk)*fse3v(ji,jj,jk)
#endif
             END DO
            END DO
         END DO
         DO jj = 1, jpj
            DO ji = 1, jpi
         	sshn_tm(ji,jj)=sshn_tm(ji,jj)+sshn(ji,jj)
                sshu_n_tm(ji,jj)=sshu_n_tm(ji,jj)+sshu_n(ji,jj)
                sshv_n_tm(ji,jj)=sshv_n_tm(ji,jj)+sshv_n(ji,jj)
	        rnf_tm(ji,jj)= rnf_tm(ji,jj)+rnf(ji,jj)
	        h_rnf_tm(ji,jj)= h_rnf_tm(ji,jj)+h_rnf(ji,jj)
             END DO
         END DO
      ELSE
         if(lwp)write(numout,*)'trc_sub_stp at kt=',kt
!it is time to substep
! 1. set temporary arrays to hold physics variables
            sshn_temp=sshn
            sshu_n_temp=sshu_n
            sshv_n_temp=sshv_n
            sshf_n_temp=sshf_n
            sshb_temp=sshb
            sshu_b_temp=sshu_b
            sshv_b_temp=sshv_b
            ssha_temp=ssha
            sshu_a_temp=sshu_a
            sshv_a_temp=sshv_a
            rnf_temp=rnf
            h_rnf_temp=h_rnf
            rotn_temp=rotn
            hdivn_temp=hdivn
            rotb_temp=rotb
            hdivb_temp=hdivb
            un_temp=un
            vn_temp=vn
            wn_temp=wn
            avt_temp=avt
#if defined key_ldfslp
            wslpi_temp=wslpi
            wslpj_temp=wslpj
            uslp_temp=uslp
            vslp_temp=vslp
#endif
            hu_temp=hu
            hv_temp=hv
            hur_temp=hur
            hvr_temp=hvr

!2. create averages and reassign variables
            DO jj = 1, jpj
               DO ji = 1, jpi
                  sshn_tm(ji,jj)= sshn_tm(ji,jj)+sshn(ji,jj)
                  sshn(ji,jj)=sshn_tm(ji,jj)/(nn_dttrc+1)
		  
                  sshu_n_tm(ji,jj)= sshu_n_tm(ji,jj)+sshu_n(ji,jj)
                  sshu_n(ji,jj)=sshu_n_tm(ji,jj)/(nn_dttrc+1)
		  
                  sshv_n_tm(ji,jj)= sshv_n_tm(ji,jj)+sshv_n(ji,jj)
                  sshv_n(ji,jj)=sshv_n_tm(ji,jj)/(nn_dttrc+1)

		  sshb(ji,jj)=sshb_hold(ji,jj)

		  sshu_b(ji,jj)=sshu_b_hold(ji,jj)

		  sshv_b(ji,jj)=sshv_b_hold(ji,jj)

                  rnf_tm(ji,jj)= rnf_tm(ji,jj)+rnf(ji,jj)
                  rnf(ji,jj)=rnf_tm(ji,jj)/(nn_dttrc+1)
 
                  h_rnf_tm(ji,jj)= h_rnf_tm(ji,jj)+h_rnf(ji,jj)
                  h_rnf(ji,jj)=h_rnf_tm(ji,jj)/(nn_dttrc+1)
               END DO
            END DO
         DO jk = 1, jpk
             IF( lk_vvl ) THEN                           !  Update Now Vertical coord.  !   (only in vvl case)
             !                                           !------------------------------!
                 fse3t (:,:,jk) = fse3t_n (:,:,jk)          ! vertical scale factors stored in fse3. arrays
                 fse3u (:,:,jk) = fse3u_n (:,:,jk)
                 fse3v (:,:,jk) = fse3v_n (:,:,jk)
                 fse3f (:,:,jk) = fse3f_n (:,:,jk)
                 fse3w (:,:,jk) = fse3w_n (:,:,jk)
                 fse3uw(:,:,jk) = fse3uw_n(:,:,jk)
                 fse3vw(:,:,jk) = fse3vw_n(:,:,jk)
             ENDIF
         ENDDO
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi

                  un_tm(ji,jj,jk)= un_tm(ji,jj,jk)+un(ji,jj,jk)*e3u_temp(ji,jj,jk)
                  un(ji,jj,jk)=(un_tm(ji,jj,jk)/(nn_dttrc+1))/fse3u(ji,jj,jk)
		  
                  vn_tm(ji,jj,jk)= vn_tm(ji,jj,jk)+vn(ji,jj,jk)*e3v_temp(ji,jj,jk)
                  vn(ji,jj,jk)=(vn_tm(ji,jj,jk)/(nn_dttrc+1))/fse3v(ji,jj,jk)
		  
                  wn_tm(ji,jj,jk)= wn_tm(ji,jj,jk)+wn(ji,jj,jk)*e3w_temp(ji,jj,jk)
                  wn(ji,jj,jk)=(wn_tm(ji,jj,jk)/(nn_dttrc+1))/fse3w(ji,jj,jk)
		  
                  avt_tm(ji,jj,jk)= avt_tm(ji,jj,jk)+avt(ji,jj,jk)*e3t_temp(ji,jj,jk)
                  avt(ji,jj,jk)=(avt_tm(ji,jj,jk)/(nn_dttrc+1))/fse3t(ji,jj,jk)	
	  
#if defined key_ldfslp
                  wslpi_tm(ji,jj,jk)= wslpi_tm(ji,jj,jk)+wslpi(ji,jj,jk)*e3w_temp(ji,jj,jk)
                  wslpi(ji,jj,jk)=(wslpi_tm(ji,jj,jk)/(nn_dttrc+1))/fse3w(ji,jj,jk)

                  wslpj_tm(ji,jj,jk)= wslpj_tm(ji,jj,jk)+wslpj(ji,jj,jk)*e3w_temp(ji,jj,jk)
                  wslpj(ji,jj,jk)=(wslpj_tm(ji,jj,jk)/(nn_dttrc+1))/fse3w(ji,jj,jk)

                  uslp_tm(ji,jj,jk)= uslp_tm(ji,jj,jk)+uslp(ji,jj,jk)*e3u_temp(ji,jj,jk)
                  uslp(ji,jj,jk)=(uslp_tm(ji,jj,jk)/(nn_dttrc+1))/fse3u(ji,jj,jk)

                  vslp_tm(ji,jj,jk)= vslp_tm(ji,jj,jk)+vslp(ji,jj,jk)*e3v_temp(ji,jj,jk)
                  vslp(ji,jj,jk)=(vslp_tm(ji,jj,jk)/(nn_dttrc+1))/fse3v(ji,jj,jk)
#endif
	       END DO
            END DO
         END DO
         CALL ssh_wzv_trc( kt )         ! after ssh & vertical velocity
      ENDIF


   END SUBROUTINE trc_sub_stp
  !!-------------------------------------------------------------------

  !!-------------------------------------------------------------------
   SUBROUTINE trc_sub_stp_ini
      !!-------------------------------------------------------------------
      !!                     ***  ROUTINE trc_sub_stp_ini  ***
      !!                      
      !! ** Purpose : Initialize variables needed for sub-stepping passive tracers
      !! 
      !! ** Method  : 
      !!              Compute the averages for sub-stepping
      !!-------------------------------------------------------------------
      INTEGER ::   jk, jn ,ji,jj   ! dummy loop indices
      INTEGER :: ierr
      !!-------------------------------------------------------------------

      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) 'trc_sub_stp_ini : initial set up of the passive tracers substepping'
      IF(lwp) WRITE(numout,*) '~~~~~~~'

      ierr =  trc_sub_alloc    ()
      IF( lk_mpp    )   CALL mpp_sum( ierr )
      IF( ierr /= 0 )   CALL ctl_stop( 'STOP', 'top_sub_alloc : unable to allocate standard ocean arrays' )

      DO jk = 1, jpk
         DO jj = 1, jpj
            DO ji = 1, jpi
               un_tm(ji,jj,jk)= un(ji,jj,jk)*fse3u(ji,jj,jk)
               vn_tm(ji,jj,jk)= vn(ji,jj,jk)*fse3v(ji,jj,jk)
	       wn_tm(ji,jj,jk)= wn(ji,jj,jk)*fse3w(ji,jj,jk)
               avt_tm(ji,jj,jk)=avt(ji,jj,jk)*fse3t(ji,jj,jk)
#if defined key_ldfslp
               wslpi_tm(ji,jj,jk)= wslpi(ji,jj,jk)*fse3w(ji,jj,jk) !0. 
               wslpj_tm(ji,jj,jk)= wslpj(ji,jj,jk)*fse3w(ji,jj,jk) !0. 
               uslp_tm(ji,jj,jk)= uslp(ji,jj,jk)*fse3u(ji,jj,jk) !0. 
               vslp_tm(ji,jj,jk)= vslp(ji,jj,jk)*fse3v(ji,jj,jk) !0. 
#endif
           END DO
         END DO
      END DO
 
      sshn_tm = sshn !0.
      sshu_n_tm = sshu_n !0.
      sshv_n_tm = sshv_n !0.
      rnf_tm= rnf !0. 
      h_rnf_tm= h_rnf !0.

   END SUBROUTINE trc_sub_stp_ini

  !!-------------------------------------------------------------------
   SUBROUTINE trc_sub_stp_reset(kt)
      !!-------------------------------------------------------------------
      !!                     ***  ROUTINE trc_sub_stp_reset  ***
      !!                      
      !! ** Purpose : Reset physics variables averaged for substepping
      !! 
      !! ** Method  : 
      !!              Compute the averages for sub-stepping
      !!-------------------------------------------------------------------
      INTEGER ::   jk, jn ,ji,jj   ! dummy loop indices
      INTEGER :: ierr
      INTEGER, INTENT( in ) ::  kt  ! ocean time-step index
      !!-------------------------------------------------------------------

      IF(lwp)write(numout,*)'trc_sub_stp_reset at kt=',kt
!restore physics variables
        avt=avt_temp
        un=un_temp
        vn=vn_temp
        wn=wn_temp
        sshn=sshn_temp
        sshb=sshb_temp
        ssha=ssha_temp
        sshu_n=sshu_n_temp
        sshu_b=sshu_b_temp
        sshu_a=sshu_a_temp
        sshv_n=sshv_n_temp
        sshv_b=sshv_b_temp
        sshv_a=sshv_a_temp
        sshf_n=sshf_n_temp
#if defined key_ldfslp
        wslpi=wslpi_temp
        wslpj=wslpj_temp
        uslp=uslp_temp
        vslp=vslp_temp
#endif
        rnf=rnf_temp
        h_rnf=h_rnf_temp
        hdivn=hdivn_temp
        rotn=rotn_temp
        hdivb=hdivb_temp
        rotb=rotb_temp
        hu=hu_temp
        hv=hv_temp
        hur=hur_temp
        hvr=hvr_temp

       !                                           !------------------------------!
       IF( lk_vvl ) THEN                           !  Update Now Vertical coord.  !   (only in vvl case)
       !                                           !------------------------------!
         DO jk = 1, jpkm1
            fsdept(:,:,jk) = fsdept_n(:,:,jk)          ! now local depths stored in fsdep. arrays
            fsdepw(:,:,jk) = fsdepw_n(:,:,jk)
            fsde3w(:,:,jk) = fsde3w_n(:,:,jk)
            !
            fse3t (:,:,jk) = fse3t_n (:,:,jk)          ! vertical scale factors stored in fse3. arrays
            fse3u (:,:,jk) = fse3u_n (:,:,jk)
            fse3v (:,:,jk) = fse3v_n (:,:,jk)
            fse3f (:,:,jk) = fse3f_n (:,:,jk)
            fse3w (:,:,jk) = fse3w_n (:,:,jk)
            fse3uw(:,:,jk) = fse3uw_n(:,:,jk)
            fse3vw(:,:,jk) = fse3vw_n(:,:,jk)
         END DO
      ENDIF

!start new averages
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  un_tm(ji,jj,jk)=un(ji,jj,jk)*fse3u(ji,jj,jk)  !0.0
                  vn_tm(ji,jj,jk)=vn(ji,jj,jk)*fse3v(ji,jj,jk)  !0.0
                  wn_tm(ji,jj,jk)=wn(ji,jj,jk)*fse3w(ji,jj,jk)  !0.0
                  avt_tm(ji,jj,jk)=avt(ji,jj,jk)*fse3t(ji,jj,jk)  !0.0
#if defined key_ldfslp
                  wslpi_tm(ji,jj,jk)= wslpi(ji,jj,jk)*fse3w(ji,jj,jk) !0. 
                  wslpj_tm(ji,jj,jk)=wslpj(ji,jj,jk)*fse3w(ji,jj,jk) !0. 
                  uslp_tm(ji,jj,jk)=uslp(ji,jj,jk)*fse3u(ji,jj,jk) !0. 
                  vslp_tm(ji,jj,jk)=vslp(ji,jj,jk)*fse3v(ji,jj,jk) !0. 
#endif
 	       END DO
            END DO
         END DO
        sshn_tm= sshn !0.0
        sshu_n_tm= sshu_n !0.0
        sshv_n_tm= sshv_n !0.0
        rnf_tm=rnf !0. 
        h_rnf_tm=h_rnf !0
        sshb_hold=sshn
        sshu_b_hold=sshu_n
        sshv_b_hold=sshv_n

   END SUBROUTINE trc_sub_stp_reset

  !!-------------------------------------------------------------------
   INTEGER FUNCTION trc_sub_alloc()
      !!-------------------------------------------------------------------
      !!                    *** ROUTINE trc_sub_alloc ***
      !!-------------------------------------------------------------------
      USE lib_mpp, ONLY: ctl_warn
      INTEGER :: ierr
      !!-------------------------------------------------------------------
      !
      ALLOCATE( un_temp(jpi,jpj,jpk)        ,  vn_temp(jpi,jpj,jpk)  ,   &
         &      wn_temp(jpi,jpj,jpk)        ,  avt_temp(jpi,jpj,jpk) ,   &
         &      sshn_temp(jpi,jpj)          ,  sshb_temp(jpi,jpj) ,    &
         &      ssha_temp(jpi,jpj)          ,  sshu_a_temp(jpi,jpj),    &
         &      sshu_n_temp(jpi,jpj)        ,  sshu_b_temp(jpi,jpj),   &
         &      sshv_n_temp(jpi,jpj)        ,  sshv_b_temp(jpi,jpj),   &
         &      sshv_a_temp(jpi,jpj)        ,  sshf_a_temp(jpi,jpj),   &
         &      sshf_n_temp(jpi,jpj)        ,  sshf_b_temp(jpi,jpj),   &
#if defined key_ldfslp
         &      wslpi_temp(jpi,jpj,jpk) ,  wslpj_temp(jpi,jpj,jpk),   &
         &      uslp_temp(jpi,jpj,jpk)  ,  vslp_temp(jpi,jpj,jpk),   &
#endif
         &      rnf_temp(jpi,jpj)           ,  h_rnf_temp(jpi,jpj) ,   &
         &      hdivn_temp(jpi,jpj,jpk)     ,  hdivb_temp(jpi,jpj,jpk),    &
         &      rotn_temp(jpi,jpj,jpk)      ,  rotb_temp(jpi,jpj,jpk),    &
         &      hu_temp(jpi,jpj)            ,  hv_temp(jpi,jpj),    &
         &      hur_temp(jpi,jpj)           ,  hvr_temp(jpi,jpj),    &
         &      un_tm(jpi,jpj,jpk)          ,  vn_tm(jpi,jpj,jpk)  ,   &
         &      wn_tm(jpi,jpj,jpk)          ,  avt_tm(jpi,jpj,jpk) ,   &
         &      sshn_tm(jpi,jpj)            ,  sshb_hold(jpi,jpj) ,    &
         &      sshu_n_tm(jpi,jpj)          ,  sshu_b_hold(jpi,jpj),   &
         &      sshv_n_tm(jpi,jpj)          ,  sshv_b_hold(jpi,jpj),   &
#if defined key_ldfslp
         &      wslpi_tm(jpi,jpj,jpk)   ,  wslpj_tm(jpi,jpj,jpk),   &
         &      uslp_tm(jpi,jpj,jpk)    ,  vslp_tm(jpi,jpj,jpk),   &
#endif
         &      rnf_tm(jpi,jpj)             ,  h_rnf_tm(jpi,jpj) ,   &
         &                                    STAT=trc_sub_alloc )  
      IF( trc_sub_alloc /= 0 )   CALL ctl_warn('trc_sub_alloc: failed to allocate arrays')

      !
   END FUNCTION trc_sub_alloc

#else
   !!----------------------------------------------------------------------
   !!   Default key                                     NO passive tracers
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE trc_sub_stp( kt )        ! Empty routine
      WRITE(*,*) 'trc_sub_stp: You should not have seen this print! error?', kt
   END SUBROUTINE trc_sub_stp
   SUBROUTINE trc_sub_stp_ini        ! Empty routine
      WRITE(*,*) 'trc_sub_stp_ini: You should not have seen this print! error?', kt
   END SUBROUTINE trc_sub_stp_ini

#endif

   !!======================================================================
END MODULE trcsubstp
